/**
	Program to calculate the cosine transform of
	an input image

	Fraga, Jul 30, 2001

**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"

/** This function calculate the Discrete Cosine Transform (DCT) 
	if kind == 0 Direct evaluation
	if kind == 1 Inverse evaluation  **/

static float *table;
static float alpha0, alpha1;
static float *buf1, *buf2;

void init_dct( int n )
{
	int i, n2;

	n2= n*2;

	table = (float *)malloc( (n2+1) * sizeof(float) );
	if ( table==NULL ) {
		fprintf( stderr, "ERROR allocating memory for coefficients table\n" );
		exit( -1 );
	}
	/** Doing table of coefficients 
		Note that table is constructed in a half of a period,
		that is from 0 to pi
	**/
	table[ 0 ] = 1.0;
	for ( i=1; i < n2; i++ )
		table[ i ] = cos ( i * M_PI / n2 ); 
	table[ n2 ] = -1.0;

	alpha0 = sqrt( 1.0/n);
	alpha1 = sqrt( 2.0/n );

	buf1 = (float *)malloc( n * sizeof(float) );
	buf2 = (float *)malloc( n * sizeof(float) );
	if ( buf1==NULL || buf2==NULL ) {
		fprintf( stderr, "ERROR allocating memory for coefficients table\n" );
		exit( -1 );
	}
}

void free_dct( )
{
	free( table );
	free( buf1 );
	free( buf2 );
	alpha0 = alpha1 = 0;
}

void dct( float *in, float *out, int n, int kind )
{
	register int u, x, j, k, s;
	int n2, k2;
	float sum;

	n2= 2*n;

	if ( !kind ) {
		sum = 0;
		for( x=0; x<n; x++ ){
			sum += in[x];
		}
		out[0] = alpha0 * sum;

		for( u=1; u<n; u++ ){
			sum = 0;
			for( x=0; x<n; x++ ){
				k = x<<1;
				k = ++k * u;
				/** Next tree lines are needed because our table was
					constructed in a half of a period :-) **/
				j = k % n2;
				s = k / n2;
				if ( s%2 ) j = n2 - j;
				sum += in[x] * table[j];
			}
			out[u] = alpha1 * sum;
		}	
	}
	else {
		for( x=0; x<n; x++ ){
			k = x<<1;
			k2 = ++k;
			sum = in[0] * alpha0;
			for( u=1; u<n; u++ ){
				k = k2 * u;
				/** Next tree lines are needed because our table was
					constructed in a half of a period :-) **/
				j = k % n2;
				s = k / n2;
				if ( s%2 ) j = n2 - j;
				sum += alpha1 * in[u] * table[j];
			}
			out[x] = sum;
		}	
	}
}

MATRIX * image_dct( MATRIX *IN, int kind )
{
	MATRIX *OUT;
	int i, j;
	int n, n2;
	float **imgin, **imgout;

	if ( IN->rows != IN->cols ) {
		fprintf( stderr, "ERROR: Input image must be a square\n" );
		return NULL;
	}
	n = IN->rows;
	n2 = 2*n; 

	OUT = matrix_allocate( n, n, sizeof(float) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return NULL;
	}


	fprintf( stderr, "Hago calculos\n" );
	imgin = (float **)IN->ptr;
	imgout = (float **)OUT->ptr;

	if ( kind == 0 ) {
		/** Column transforms **/
		for ( j=0; j<n; j++ ) {
			for ( i=0; i<n; i++ ) buf1[i] = imgin[i][j];

			dct ( buf1, buf2, n, 0 ); 

			for ( i=0; i<n; i++ ) imgout[i][j] = buf2[i];
		}
		/** Row transforms **/
		for ( i=0; i<n; i++ ) {
			dct ( imgout[i], buf2, n, 0 ); 

			for ( j=0; j<n; j++ ) imgout[i][j] =  buf2[j];
		}
	}
	else {
		/** Row transforms **/
		for ( i=0; i<n; i++ ) 
			dct ( imgin[i], imgout[i], n, 1 ); 
		
		/** Column transforms **/
		for ( j=0; j<n; j++ ) {
			for ( i=0; i<n; i++ ) buf1[i] = imgout[i][j];

			dct ( buf1, buf2, n, 1 ); 

			for ( i=0; i<n; i++ ) imgout[i][j] = buf2[i];
		}
	}
	return OUT;	
}

