/**
	Functions to calculate the hartley transform of
	an input image

**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"
#include "tiff.h"

static float *table;
static float *buf1, *buf2;

void rescale( float **imgin, byte **imgout, int rows, int cols )
{   
	float fmin, fmax, val, scale;
	int i, j;

	/** We rescale the dinamic range from 0 to 255 **/
	fmin = 1e30;
	fmax = -1e30;
	for ( i=0; i<rows; i++ ){
		for ( j=0; j<cols; j++ ){
			val = imgin[i][j]; 
			if ( fmin > val ) fmin = val;
			if ( fmax < val ) fmax = val;
		}   
	}
	printf( "max = %f\n", fmax );
	printf( "min = %f\n", fmin );

	scale = 255.0/(fmax-fmin);
	for ( i=0; i<rows; i++ ) {
		for ( j=0; j<cols; j++ ) {
			imgout[i][j] = (byte)(scale*( imgin[i][j] - fmin ) + 0.5 );
		}
	}
}

void init_dht( int n )
{
	int i;
	float sqrt2, pi2_n;

	table = (float *)malloc( n * sizeof(float) );
	if ( table==NULL ) {
		fprintf( stderr, "ERROR allocating memory for coefficients table\n" );
		exit( -1 );
	}
	sqrt2 = sqrt( 2.0 );
	pi2_n = (2.0 * M_PI) / n;
	/** Doing table of coefficients **/
	table[ 0 ] = 1.0;
	for ( i=1; i < n; i++ )
		table[ i ] = sqrt2 * cos ( i * pi2_n - M_PI_4 ); 

	buf1 = (float *)malloc( n * sizeof(float) );
	buf2 = (float *)malloc( n * sizeof(float) );
	if ( buf1==NULL || buf2==NULL ) {
		fprintf( stderr, "ERROR allocating memory for coefficients table\n" );
		exit( -1 );
	}
}

void free_dht( )
{
	free( table );
	free( buf1 );
	free( buf2 );
}

void dht( float *in, float *out, int n )
{
	register int t, v;
	float sum;

	sum = 0.0;
	for( t=0; t<n; t++ ){
		sum += in[t];
	}
	out[0] = sum;

	for( v=1; v<n; v++ ){
		sum = 0.0;
		for( t=0; t<n; t++ ){
			sum += in[t] * table[(t*v)%n];
		}
		out[v] = sum;
	}	
}

MATRIX * image_dht( MATRIX *IN )
{
	MATRIX *OUT;
	int i, j;
	int n, n2;
	float a, b, c, d, sum;
	float **imgin, **imgout;

	if ( IN->rows != IN->cols ) {
		fprintf( stderr, "ERROR: Input image must be a square\n" );
		return NULL;
	}
	n = IN->rows;
	n2 = n/2; 

	OUT = matrix_allocate( n, n, sizeof(float) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return NULL;
	}


	fprintf( stderr, "Hago calculos\n" );
	imgin = (float **)IN->ptr;
	imgout = (float **)OUT->ptr;

	/** Row transforms **/
	for ( i=0; i<n; i++ ) {
		dht ( imgin[i], buf2, n);

		for ( j=0; j<n; j++ ) imgout[i][j] =  buf2[j];
	}
	/** Column transforms **/
	for ( j=0; j<n; j++ ) {
		for ( i=0; i<n; i++ ) buf1[i] = imgout[i][j];

		dht ( buf1, buf2, n );

		for ( i=0; i<n; i++ ) imgout[i][j] = buf2[i];
	}
	/** Oscarization */
	for ( i=1; i<n2; i++ ) {
		for ( j=1; j<n2; j++ ) {
			a = imgout[i][j];
			b = imgout[i][n-j];
			c = imgout[n-i][j];
			d = imgout[n-i][n-j];
			sum = (a+b+c+d)/2;
			imgout[i][j] = sum - d;
			imgout[i][n-j] = sum - c;
			imgout[n-i][j] = sum - b;
			imgout[n-i][n-j] = sum - a;
		}
	}

	return OUT;
}

