/**
	This program realice a low-pass filtering
	in the frequency domain using the
	Discrete Hartley Transform
**/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void hartleyfilter( float **img, int rows, int cols, int r, int type )
{
	int i, j;
	int c2=cols/2;
	int r2 = r*r;
	int r_1, r_2, r_12, r_22, radio2;

	float cx, cy, x, y, val, y2, aux, aux2;

	cy = (rows-1)/2.0;
	cx = (cols-1)/2.0;
	
	switch ( type ) {
		case 0:  /** Ideal Low pass **/
		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				if ( x*x + y2 > r2 ) {
					img[i][j] = 0; 
				}
			}
		}
		break;

		case 1:   /** Ideal High pass **/
		val = img[c2][c2];
		for (i=0; i<rows ; i++) {
			y = i - cx;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				if ( x*x + y2 <= r2 ) {
					img[i][j] = 0; 
				}
			}
		}
		img[c2][c2] = val;
		break;
		
		case 2: /** Raised Cosine Low Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = M_PI/(r_2-r_1);
		r_12 = r_1*r_1;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ); /** Do nothing **/ 
				else if (radio2 < r_22 ) {
					aux2 = (1.+ cos((sqrt(radio2)-r_1)*aux))*0.5;
					img[i][j] *= aux2; 
				}
				else {
					img[i][j] = 0; 
				}
			}
		}
		break;

		case 3: /** Raised Cosine High Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = M_PI/(r_2-r_1);
		r_12 = r_1*r_1;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ) {
					img[i][j] = 0; 
				}
				else if (radio2 < r_22 ) {
					aux2 = (1. - cos((sqrt(radio2)-r_1)*aux))*0.5;
					img[i][j] *= aux2; 
				}
			}
		}
		break;

		case 4: /** Gauss Low Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		aux = (r-r_1)/0.8325546112;  /** sqrt(-ln(-1.5)) **/
		r_12 = r_1*r_1;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ); /** Do nothing **/ 
				else {
					aux2 = (sqrt(radio2)-r_1)/aux;
					aux2 = exp(-aux2*aux2);
					img[i][j] *= aux2; 
				}
			}
		}
		break;

		case 5: /** Gauss High Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = (r-r_1)/0.8325546112;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if (radio2 < r_22 ) {
					aux2 = (sqrt(radio2)-r_2)/aux;
					aux2 = exp(-aux2*aux2);
					img[i][j] *= aux2; 
				}
			}
		}
		break;

		case 6: /** Fermi Low Pass **/
		aux = 0.05;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				aux2 = 1/(1 + exp((sqrt(radio2)-r)/aux));
				img[i][j] *= aux2; 
			}
		}
		break;

		case 7: /** Fermi High Pass **/
		default:
		aux = -0.05;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=cols;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				aux2 = 1/(1 + exp((sqrt(radio2)-r)/aux));
				img[i][j] *= aux2; 
			}
		}
		break;

	}
}
