#include <stdio.h>
#include <stdlib.h>
#include "matrix.h"


#define BACKTONE	255
#define OBJTONE		0
#define FILLTONE	10

#define UP   	0
#define DOWN	1
#define E1  	2
#define E2  	3
#define NPARAM 	4

/** Variables for STACK **/

typedef int typexy;
typedef unsigned char uchar;

#define STACK_SIZE 300

typedef struct {
	typexy a, b;
} POS;

POS stack[ STACK_SIZE ];
int pos_stack = -1; 

/********** Funtiones for the stack ******************/
int push( int a, int b )
{
	int pone;
 
	pone = pos_stack;
	pone++;
	if ( pone >= STACK_SIZE ){
		fprintf( stderr, "Stack is Full %d!\n", pone );
		exit(1);
	}
	stack[pone].a = a;
	stack[pone].b = b;
	pos_stack = pone;
	return( 1 );      /** ok! **/
}

int pop( int *a, int *b )
{
	if ( pos_stack<0 ) return(0);   /** Error **/
	*a = stack[pos_stack].a;
	*b = stack[pos_stack].b;
	pos_stack--;
	return( 1 );    /** Ok! **/
}

int empty_stack( void )
{
	if( pos_stack<0 ) return( 0 ); /** It's empty **/
	return( 1 );                    /** There is something **/
}

void clean_stack( void )
{
	pos_stack = -1;
}

int fill ( MATRIX *IMG, int x, int y )
{
	byte **img;

	img = (byte **)IMG->ptr;
	while ( img[y][x] == OBJTONE ) {
		img[y][x++] = FILLTONE; 
		if ( x==IMG->cols ) return( 0 );
	}
	return(1);
}

int left ( MATRIX *IMG, int x, int y ) 
{
	byte **img;

	img = (byte **)IMG->ptr;
	while ( img[y][x] == OBJTONE ) {
		--x; 
		if ( x<0 ) return( -1 );	/** Error!!! **/
	}
	return ( ++x );
}

int link( MATRIX *IMG, int x, int y, int *par  )
{
	int above=0, below=0;
	int e1, e2 = -1;
	byte **img;

	img = (byte **)IMG->ptr;

	if ( img[y-1][x-1]==OBJTONE ) {
		above++;
		e1 = left( IMG, x-1, y-1 );
		if ( e1<0 ) return( 0 );
	}
	if ( img[y+1][x-1]==OBJTONE ) {
		below++;
		e2 = left( IMG, x-1, y+1 );
		if ( e2<0 ) return( 0 );
	}
	while ( img[y][x] == OBJTONE ) {
		if ( img[y-1][x-1]==BACKTONE && img[y-1][x]==OBJTONE ) {
			if ( above ) push ( e1, y-1 );
			above++;
			e1 = left( IMG, x, y-1 );
			if ( e1<0 ) return( 0 );
		}
		if ( img[y+1][x-1]==BACKTONE && img[y+1][x]==OBJTONE ) {
			if ( below ) push ( e2, y+1 );
			below++;
			e2 = left( IMG, x, y+1 );
			if ( e2<0 ) return( 0 );
		}
		x++;
		if ( x==IMG->cols ) return( 0 );
	}
	if ( img[y-1][x-1]==BACKTONE && img[y-1][x]==OBJTONE ){
		if ( above ) push ( e1, y-1 );
		above++;
		e1 = left( IMG, x, y-1 );
		if ( e1<0 ) return( 0 );
	}
	if ( img[y+1][x-1]==BACKTONE && img[y+1][x]==OBJTONE ){
		if ( below ) push ( e2, y+1 );
		below++;
		e2 = left( IMG, x, y+1 );
		if ( e2<0 ) return( 0 );
	}
	par[UP] = above;
	par[DOWN] = below;
	par[E1] = e1;
	par[E2] = e2;
	return( 1 );
}

int do_fill( MATRIX *IMG )
{
	int x, y;
	int param[ NPARAM ];
	int pnext, dir;
	byte **img;

	img = (byte **)IMG->ptr;
	dir = UP;
	while( pop( &x, &y ) ){
		for(;;){
			if ( !link( IMG, x, y, param ) ) return(0);
			if ( !param[UP] && !param[DOWN] ){
				if ( img[y][x]==OBJTONE ) 
					if( !fill( IMG, x, y ) ) return(0);
				break;
			}

			if ( !param[UP] ) dir=DOWN;
			else if ( !param[DOWN] ) dir=UP;

			if ( param[UP] && param[DOWN] ) {
				if( dir==UP ) push ( param[ E1+DOWN ], y+1 );
				else  push ( param[ E1+UP ], y-1 );
			}
			pnext = param[ E1+dir ];

			if ( !fill( IMG, x, y ) ) return(0);

			x = pnext;
			y = (dir==UP) ? y-1 : y+1;
			if ( y<0 || y==IMG->rows ) return(0);
		}
	}
	return( 1 );
} 
