#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "pnm.h"

typedef unsigned short int tipoxy;
    /** Sirve para almacenar hasta imagenes de 255
    	pixeles de ancho o largo. Para imagenes mas grandes
    	se cambiara' a short int **/

typedef struct {
	tipoxy *x, *y;
} P_FRANJA;		/** 3 bytes/punto **/

P_FRANJA puntos;

int Porden[10];

/**** Variables para el stack *****/
#define TAM_STACK	20
typedef struct{
	int a, b;
} POS;

POS stack[ TAM_STACK ];
int pos_stack = -1;
/**********************************/

/**** Cola para almacenar las franjas *****/
typedef unsigned short int usint;
struct nodo{
	usint inicio;
	int largo;
	struct nodo *sig;
};

typedef struct nodo NODO;

typedef struct {
	NODO *prpio, *final;
} COLA;	

COLA franjas; /** Variable global **/
COLA resultado; /** Variable global **/
/***********************************************/

/***** Tabla para el algoritmo del rastreo..
	Usada por la funcion me_voy()          ***/
const int tabla[8][2] = {
/** 0 **/ { 0, -1},
/** 1 **/ { 1, -1},
/** 2 **/ { 1,  0},
/** 3 **/ { 1,  1},
/** 4 **/ { 0,  1},
/** 5 **/ {-1,  1},
/** 6 **/ {-1,  0},
/** 7 **/ {-1, -1} };
/***********************************************/
#define TRUE	1
#define FALSE	0

/********** Funciones para el stack  ********************/
int push( int a, int b )
{
	int pone;

	pone = pos_stack;
	pone++;
	if ( pone >= TAM_STACK ) return( 1 );  /** Error **/
	stack[pone].a = a;
	stack[pone].b = b;
	pos_stack = pone;
	return( 0 );	  /** ok! **/
}

int pop( int *a, int *b )
{
	if ( pos_stack<0 ) return(1);   /** Error **/ 
	*a = stack[pos_stack].a;
	*b = stack[pos_stack].b;
	pos_stack--;
	return( 0 ); 	/** Ok! **/
}

int empty_stack( )
{
	if( pos_stack<0 ) return( 1 ); /** Esta vacio **/
	return( 0 );                    /** Hay algo **/
}
/********************************************************/

/********** Funciones para la cola de franjas   ********************/
NODO *crea_nodo( )
{
	NODO *p;
	
	p = (NODO *) malloc( sizeof(NODO) );
	if( p==NULL ){
		printf("No hay memoria para los nodos\n");
		exit( 1 );
	}
	p->inicio = 0;
	p->largo = -1;
	p->sig = NULL;
	return( p );
}

void pone_nodo( NODO *p )
{
	if ( franjas.prpio==NULL ){
		franjas.prpio = p;
		franjas.final = p;
	}
	else {
		franjas.final->sig = p;
		franjas.final = p;
	}
}

NODO * borra_nodo( NODO *p )
{
	NODO *paux1, *paux2;
	
	if( franjas.prpio == franjas.final ) {
		paux1 = franjas.prpio;
		franjas.prpio = NULL;
		franjas.final = NULL;
	}
	else if ( p == franjas.prpio ) {
		paux1 = franjas.prpio;
		franjas.prpio = franjas.prpio->sig;
	}
	else{
		paux2 = franjas.prpio;
		paux1 = paux2 -> sig;
		while( paux1 != NULL ) {
			if ( paux1 == p ) {
				paux2->sig = paux1->sig;
				break;
			}
			paux2 = paux2 -> sig;
			paux1 = paux2 -> sig;
		}
		if ( paux1 == franjas.final )
			franjas.final = paux2;
	}
	paux1->sig = NULL;
	return( paux1 );
}
		
		
void pone_nodo_resultado( NODO *p )
{
	if ( resultado.prpio==NULL ){
		resultado.prpio = p;
		resultado.final = p;
	}
	else {
		resultado.final->sig = p;
		resultado.final = p;
	}
}

void borra_cola_nodos( )
{
	NODO *paux1, *paux2;

	paux1 = franjas.prpio;
	paux2 = paux1->sig;

	if( paux1 == NULL ) return;
	while( paux2 != NULL ){
		free( paux1 );
		paux1=paux2;
		paux2 = paux1->sig;
	}
	free( paux1 );
	franjas.prpio = NULL;
	franjas.final = NULL;
}
		

/********************************************************/

long cuenta_puntos( MATRIX *IN1 )
{
    int i, j;
    long N=0L;
    unsigned char *inptr;

    for ( i=2; i < (IN1->rows)-2; i++ ) {
        inptr = (unsigned char *) IN1->ptr[i];
        for ( j = 2; j < (IN1->cols)-2 ; j++ ){
            if ( *inptr ) N++;
            inptr++;
        }
    }    
    return( N*sizeof(tipoxy) );
}

void localiza_memoria( MATRIX *IMA )
{
	long bytes; 

	bytes = cuenta_puntos( IMA );
	printf("Puntos = %ld\n", bytes );
	if ( bytes > 65500 ) {
		printf("Demasiados puntos para reservar memoria!\n");
		exit( 1 );
	}
	puntos.x = (tipoxy *) malloc( bytes );
	puntos.y = (tipoxy *) malloc( bytes );
	if ( puntos.x==NULL || puntos.y==NULL ) {
		printf("No hay memoria para los puntos de las franjas\n");
		exit( 1 );
	}
}
	
int checa_punto( MATRIX *IMA, int row, int col )
{
	static int Pt[9];
    int *ptrP, *paux;
    int  i, j, k, sp;
	unsigned char *ptrIMA;
	
	row--;
	col--;
	k=0;
	for( i=0; i<3; i++ ){
		ptrIMA = (unsigned char *)IMA->ptr[ row+i ];
		ptrIMA += col;
		for( j=0; j<3; j++ ){
			Pt[k] = (*ptrIMA>0) ? 1 : 0;
			k++; ptrIMA++;
		}
	}
	Porden[0] = Pt[4]; Porden[1] = Pt[1]; Porden[2] = Pt[2];
	Porden[3] = Pt[5]; Porden[4] = Pt[8]; Porden[5] = Pt[7];
	Porden[6] = Pt[6]; Porden[7] = Pt[3]; Porden[8] = Pt[0];
	Porden[9] = Pt[1];

	/** No. de transiciones de 0-1 **/
	ptrP = Porden + 1;
	paux = ptrP + 1;
	sp = 0;
	for( i=0; i<8; i++ ) {
		if( *ptrP==0 && *paux>0 ) sp++;
		ptrP++;
		paux++;
	}
	return( sp );
}
 

void me_voy( int *row, int *col )
{
	int i=-1;

	if ( Porden[1] ) i=0;
	else if ( Porden[3] ) i=2;
	else if ( Porden[5] ) i=4; 
	else if ( Porden[7] ) i=6;
	
	if ( i < 0 ){
		if ( Porden[2] ) i=1;
		else if ( Porden[4] ) i=3;
		else if ( Porden[6] ) i=5;
		else if ( Porden[8] ) i=7;
	}

	*row += tabla[i][1];
	*col += tabla[i][0];
}   

void halla_franjas( MATRIX * IN ) 
{
	int col, row;
	usint donde_inicia;
	int  cuenta;
	int termina_franja;
	int cx, cy;
	int sp, flag;
	NODO *pnodo;
	unsigned char **IMA;
	int numero=0;
	
	IMA = (unsigned char **)IN->ptr;
	
	localiza_memoria( IN );

	donde_inicia = 0; /** Indice de los arrays de la
                         estructura franjas **/
 
	for( row=2; row<IN->rows-2; row++ ){
		for( col=2; col<IN->cols-2; col++ ){
			if( IMA[row][col] ) {  /** IN[row][col] > 0 **/
				/** Inicializacion de una franja **/
				pnodo = crea_nodo( );
				pnodo->inicio=donde_inicia;
				cx = col; cy = row;
				cuenta = 0;
				numero++;
			}
			else continue;
			termina_franja = FALSE;
			flag = 1;
			do{
				if( IMA[cy][cx] == 0 && flag ){
					printf("ERROR no es blanco!!!\n");
					exit(1);
				}
				
				sp = checa_punto( IN, cy, cx );
				
				if ( flag ) {	
					/** Pone el punto en la lista **/
					puntos.x[donde_inicia] = (tipoxy) cx;
					puntos.y[donde_inicia] = (tipoxy) cy;
					donde_inicia++; cuenta++;

					IMA[cy][cx] = 0;	/** Lo borro **/
					flag = 1;
				}

				if ( sp == 1 ) me_voy( &cy, &cx );
				else if (!sp) {
					if( empty_stack( ) ){    /** Esta vacio **/
						termina_franja = TRUE;
					}
					else {
						pop ( &cy, &cx );
						flag = 0;
					}
				}
				else {
					if ( push( cy, cx ) ) {
						printf("Error en PUSH 1\n");
						printf("loc = %d\n", pos_stack );
						exit(1);
					}
					me_voy( &cy, &cx );
				}
			} while( termina_franja == FALSE );

			pnodo->largo = cuenta;	
			pone_nodo( pnodo );
	/**		printf("Puntos=%3d, Termine %d franja\n", cuenta, tt );
	**/
		}
	}	
	printf("Encontre %d franjas\n", numero );
	/** matrix_free( IN ); **/
}



/**
	Retorna el nmero de franjas que se borraron
**/
int borra_franjas_menores( MATRIX *IMG, int min )
{
	int i=0, j, largo;
	usint base, loc;
	NODO *paux, *paux1;
	byte **img;

	img = (byte **) IMG->ptr;

	paux = franjas.prpio;
	if ( paux == NULL ) return( 1 );	/** Error **/
	while( paux != NULL ){
		if( paux->largo <= min ) {
			paux1 = paux -> sig;
			i++;
			largo = paux -> largo;
			base = paux -> inicio;
			for( j=0; j<largo; j++ ){
				loc = base + (usint) j;
				img[ puntos.y[loc]][ puntos.x[loc]] = 0;
			}
			borra_nodo( paux );
			paux = paux1;
		}
		else
			paux = paux -> sig;
	}
	return( i );
}	

void guarda_datos ( int rows, int cols  )
{
	FILE *fp;
	NODO * paux;
	int i, k, largo;
	usint base, loc;

	if ( (fp=fopen( "franjas.txt", "w" ))==NULL ){
		printf("Error abriendo archivo franjas.txt\n");
		exit(1);
	}
	/** Guardamos el ancho y largo de la imagen procesada **/
	fprintf( fp, "%d  %d\n", rows, cols ); 

	/** Contamos el nmero de franjas y lo guardamos **/	
	paux = franjas.prpio;
	k = 0;
	while( paux != NULL ){
		k++;
		paux = paux->sig;
	}
	fprintf( fp, "%d\n", k ); 

	paux = franjas.prpio;
	k = 0;
	while( paux != NULL ){
		k++;
		largo = paux -> largo;
		base = paux -> inicio; 
		fprintf( fp, "%d\n", largo );
		for( i=0; i<largo; i++ ){
			loc = base + (usint) i;
			fprintf( fp, "%d  %d\n", puntos.x[loc], puntos.y[loc] ); 
		} 
		paux = paux->sig;
	}
	fclose( fp );

}

int main( int argc, char *argv[] )
{
	MATRIX *IMG, *IMG_AUX;
	byte *ren, *ren_aux;
	char path[80];
	int i, j;

	if ( argc < 2 ) {
		printf("Uso: %s img_entrada.pbm img_salida.pbm\n", argv[0]);
		exit(1);
	}

	strncpy( path, argv[1], 80 );
	printf ( "Se lee la imagen de entrada %s\n", path );
	IMG = read_pnm( path );
	if ( IMG == NULL ){
		fprintf ( stderr, "Error en el archivo de imagen de entrada %s\n", path );
		return( -1 );
	}
	
	/* Se necesita una imagen auxiliar para no perder la imagen 
	   original pues al extraer las franjas, stas se van borrando
	   de la imagen.
	*/
	IMG_AUX = matrix_allocate( IMG->rows, IMG->cols, 1 );
	for( i=0; i<IMG->rows; i++ ){
		ren     = (byte *)IMG->ptr[i];
		ren_aux = (byte *)IMG_AUX->ptr[i];
		for( j=0; j<IMG->cols; j++ )
			ren_aux[j] = ren[j];
	}
	halla_franjas( IMG_AUX );

	borra_franjas_menores( IMG, 10 );

	guarda_datos( IMG->rows, IMG->cols );

	strncpy( path, argv[2], 80 );
	IMG->type = 1;
	write_pnm( IMG, path );

	borra_cola_nodos( );
	matrix_free(IMG);
	matrix_free(IMG_AUX);
/**
	free( puntos.x );
	free( puntos.y );
**/

	return ( 0 );
}

