/**
	Program to thresholding an imagen

	Fraga. Jan 21, 2009
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"

int do_work( char *namein, char *nameout, int thr1, int thr2 ) 
{
	MATRIX *IN, *OUT;
	int i, j;
	byte *pi, *po;
	

	IN = read_pnm( namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}

	OUT = matrix_allocate( IN->rows, IN->cols, sizeof( byte ) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory\n" );
		return -1;
	}

	for( i=0; i<IN->rows; i++ ){
		pi = (byte *) IN -> ptr[i];
		po = (byte *) OUT -> ptr[i];
		for( j=0; j<IN->cols; j++ ){
			if( thr1 < *pi && *pi < thr2 ) *po = 255; /** 0xff **/
			else  *po = 0;
			pi++;
			po++;
		}
	}

	if ( write_pnm( OUT, nameout ) != 0 )  
        return -1;

	matrix_free( OUT );
	matrix_free( IN );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];
	char nameout[32];
	int val1, val2;

	if ( argc < 5 ) { 
		fprintf( stderr, "Use: %s input_image.pgm output_image.pgm threshold1 threshold2\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	strncpy( nameout, argv[2], 32 );

	val1 = strtol( argv[3], (char **) NULL, 10);
	val2 = strtol( argv[4], (char **) NULL, 10);

	if( val2 < val1 ) {
		fprintf( stderr, "ERROR: thr1 must less than thr2!\n" );
		return 1;
	}

	if ( do_work( namein, nameout, val1, val2 ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
